<?php
namespace PME\Controllers\Cardapio;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;

class UtilitiesCardapio extends \PME\Services\TeknisaCrud {
    protected $environment;
    protected $connection;
    protected $newCode;
    
    public function __construct($crudService, $parameterBag, $environment, $connection, $newCode) {
      $this->environment = $environment;
      $this->connection = $connection;
      $this->newCode = $newCode;
      parent::__construct($crudService, $parameterBag);
    }
    
    public function getCadiaPme($central, $nrcardpme, $servico, $dtcardiapme, $filial) {
        $sql = "SELECT CDFILCENTPME,
                  NRCARDPME,
                  NRCARDIAPME,
                  CDSERVFILI,
                  CDFILIAL,
                  DTCARDIAPME,
                  QTCOMESTCPME
                FROM CARDIAPME
                WHERE CDFILCENTPME = :P_CDFILCENTPME
                AND NRCARDPME      = :P_NRCARDPME
                AND CDSERVFILI     = :P_CDSERVFILI
                AND CDFILIAL       = :P_CDFILIAL
                AND TRUNC(DTCARDIAPME)    = TRUNC(TO_DATE(:P_DTCARDIAPME))";
		$paramSelect = array(
            "P_CDFILCENTPME" => $central,
            "P_NRCARDPME"    => $nrcardpme,
            "P_CDSERVFILI"   => $servico,
            "P_DTCARDIAPME"  => substr($dtcardiapme, 0, 10),
            "P_CDFILIAL"     => $filial
        );
        
        $stmt = $this->connection->prepare($sql);
		$stmt->execute($paramSelect);
		
		return $stmt->fetchAll();
    }
    
    public function updateCardiaPme($cardespec, $central, $nrcardpme, $nrcardiapme, $servico, $dtcardiapme, $filial) {
        $sql = "UPDATE CARDIAPME SET IDCARDESPEC = :P_IDCARDESPEC WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND NRCARDIAPME = :P_NRCARDIAPME AND CDSERVFILI = :P_CDSERVFILI AND CDFILIAL = :P_CDFILIAL /*AND DTCARDIAPME = :P_DTCARDIAPME*/";
		$paramSelect = array(
            "P_IDCARDESPEC"  => $cardespec,
            "P_CDFILCENTPME" => $central,
            "P_NRCARDPME"    => $nrcardpme,
            "P_NRCARDIAPME"  => $nrcardiapme,
            "P_CDSERVFILI"   => $servico,
            // "P_DTCARDIAPME"  => $dtcardiapme,
            "P_CDFILIAL"     => $filial
        );
        
        $stmt = $this->connection->prepare($sql);
		$stmt->execute($paramSelect);
    }
    
    public function deleteCardia($cardia) {
        $delete = "DELETE FROM CARDIAPME WHERE CDFILCENTPME = :CDFILCENTPME AND NRCARDPME = :NRCARDPME AND NRCARDIAPME = :NRCARDIAPME AND CDSERVFILI = :CDSERVFILI AND CDFILIAL = :CDFILIAL AND DTCARDIAPME = :DTCARDIAPME";
        $params = array(
            "CDFILCENTPME"  => $cardia['CDFILCENTPME'],
            "NRCARDPME"     => $cardia['NRCARDPME'],
            "NRCARDIAPME"   => $cardia['NRCARDIAPME'],
            "CDSERVFILI"	=> $cardia['CDSERVFILI'],
            "CDFILIAL"	    => $cardia['CDFILIAL'],
            "DTCARDIAPME"	=> substr($cardia['DTCARDIAPME'], 0, 10)
        );
        
        $stmt = $this->connection->prepare($delete);
    	$stmt->execute($params);
    } 
    
    public function deleteCompCard($central, $nrcardpme, $nrcardiapme, $nrseqpracpme, $tpPrato) {
        $delete = "DELETE FROM COMPCARDPME WHERE CDFILCENTPME = :CDFILCENTPME AND NRCARDPME = :NRCARDPME AND NRCARDIAPME = :NRCARDIAPME AND NRSEQPRACPME = :NRSEQPRACPME AND CDTIPOPRAT = :CDTIPOPRAT";
        $params = array(
            "CDFILCENTPME"  => $central,
            "NRCARDPME"     => $nrcardpme,
            "NRCARDIAPME"   => $nrcardiapme,
            "NRSEQPRACPME"	=> $nrseqpracpme,
            "CDTIPOPRAT"	=> $tpPrato
        );
        
        $stmt = $this->connection->prepare($delete);
    	$stmt->execute($params);
    }
    
    public function insertCompCard($cdfilcentpme, $nrcardpme, $nrcardiapme, $nrseqcnfcpm, $cdfilauxprat, $cdprat, $cdtipoprat, $vrpeconscpm, $nrordcnfcpme) {
        $insertPlate = "INSERT INTO COMPCARDPME (CDFILCENTPME, NRCARDPME, NRCARDIAPME, NRSEQPRACPME, CDFILAUXPRAT, CDPRATO, CDTIPOPRAT, VRPECONSCPME, NRORDPRACPME) VALUES (:CDFILCENTPME, :NRCARDPME, :NRCARDIAPME, :NRSEQPRACPME, :CDFILAUXPRAT, :CDPRATO, :CDTIPOPRAT, :VRPECONSCPME, :NRORDPRACPME)";
        $params = array(
            "CDFILCENTPME" => $cdfilcentpme,
            "NRCARDPME"    => $nrcardpme,
            "NRCARDIAPME"  => $nrcardiapme,
            "NRSEQPRACPME" => $nrseqcnfcpm,
            "CDFILAUXPRAT" => $cdfilauxprat,
            "CDPRATO"      => $cdprat,
            "CDTIPOPRAT"   => $cdtipoprat,
            "VRPECONSCPME" => $vrpeconscpm,
            "NRORDPRACPME" => $nrordcnfcpme
        );
        $stmt = $this->connection->prepare($insertPlate);
        $stmt->execute($params);
    }
}