CREATE OR REPLACE PROCEDURE LIMPA_ITEMAUTF_ORDERNUM
IS
BEGIN 
    FOR C_AUTOFORNE IN (
        SELECT CDFILDEST,
               CDFILLANC,
               NRAUTOFORN
          FROM AUTOFORNE
         WHERE USR_DTEXPOAUTOFORN IS NULL
           AND USR_NRAUTOFORN IS NULL
           AND EXISTS (SELECT 1
                         FROM ITEMAUTF
                        WHERE ITEMAUTF.CDFILDEST = AUTOFORNE.CDFILDEST
                          AND ITEMAUTF.CDFILLANC = AUTOFORNE.CDFILLANC
                          AND ITEMAUTF.NRAUTOFORN = AUTOFORNE.NRAUTOFORN
                          AND USR_ORDERNUM IS NOT NULL)) 
    LOOP 
        UPDATE ITEMAUTF 
           SET USR_ORDERNUM = NULL 
         WHERE CDFILDEST = C_AUTOFORNE.CDFILDEST
           AND CDFILLANC = C_AUTOFORNE.CDFILLANC
           AND NRAUTOFORN = C_AUTOFORNE.NRAUTOFORN;
        COMMIT;
    END LOOP; 
END;
/

BEGIN
   DECLARE
      V_COUNT NUMBER;
   BEGIN
      V_COUNT := 0;
      BEGIN
         SELECT COUNT(*)
            INTO V_COUNT
            FROM USER_SCHEDULER_JOBS
           WHERE JOB_NAME = 'JOB_LIMPA_ITEMAUTF_ORDERNUM';
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_COUNT := 0;
      END;
      IF V_COUNT = 0 THEN
         DBMS_SCHEDULER.CREATE_JOB (
            job_name        => 'JOB_LIMPA_ITEMAUTF_ORDERNUM',
            job_type        => 'PLSQL_BLOCK',
            job_action      => 'BEGIN LIMPA_ITEMAUTF_ORDERNUM(); END;',
            start_date      => SYSDATE,
            repeat_interval => 'FREQ=MINUTELY;INTERVAL=10',
            end_date        => NULL,
            enabled         => TRUE,
            comments        => 'Job criado para corrigir erro na integração de itens de AF.'
         );
      ELSE
         DBMS_SCHEDULER.DROP_JOB(JOB_NAME => 'JOB_LIMPA_ITEMAUTF_ORDERNUM');
         DBMS_SCHEDULER.CREATE_JOB (
            job_name        => 'JOB_LIMPA_ITEMAUTF_ORDERNUM',
            job_type        => 'PLSQL_BLOCK',
            job_action      => 'BEGIN LIMPA_ITEMAUTF_ORDERNUM(); END;',
            start_date      => SYSDATE,
            repeat_interval => 'FREQ=MINUTELY;INTERVAL=10', 
            end_date        => NULL,
            enabled         => TRUE,
            comments        => 'Job criado para corrigir erro na integração de itens de AF.'
         );
      END IF;
   END;
END;
/