<?php
namespace Teknisa\Model\Gestaopessoas\Repositories;

use HCM\Util\FPA;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;

class FpaCalculoconsumer extends \Doctrine\ORM\EntityRepository {
    
    const IDSITUACAO_PENDENTE  = 'PENDENTE';
	const IDSITUACAO_ANDAMENTO = 'ANDAMENTO';
	const IDSITUACAO_CONCLUIDO = 'CONCLUIDO';

	public function addNrtotprocessvinc($nrorg, $nrcalculoconsumer) {
		$sql = "UPDATE FPA_CALCULOCONSUMER 
				   SET NRTOTPROCESSVINC = NVL(NRTOTPROCESSVINC, 0) + 1 
				 WHERE NRORG             = $nrorg 
				   AND NRCALCULOCONSUMER = $nrcalculoconsumer";

        \Setforms2\Shared\Registry::getEntityManagerConnection()->exec($sql);
	}

	public function concluirCalculoConsumer($nrorg, $nrcalculoconsumer) {
		$idsituacao = self::IDSITUACAO_CONCLUIDO;
		$dthrfim    = DateUtil::getDataAtual();

		$sql = "UPDATE FPA_CALCULOCONSUMER 
	   			   SET IDSITUACAO = '$idsituacao',
	   			       DTHRFIM    = '$dthrfim' 
	   			 WHERE NRORG             = $nrorg
	   			   AND NRCALCULOCONSUMER = $nrcalculoconsumer
	   			   AND NRTOTVINCULOS     = NRTOTPROCESSVINC";

        \Setforms2\Shared\Registry::getEntityManagerConnection()->exec($sql);	
	}

    public function updateDspidprocesso($nrorg, $nrcalculoconsumer, $pid) {
        $sql = "UPDATE FPA_CALCULOCONSUMER
				   SET DSPIDPROCESSO = DECODE(DSPIDPROCESSO, NULL, TO_CHAR($pid), DSPIDPROCESSO || ',' || TO_CHAR($pid))
				 WHERE NRORG             = $nrorg
				   AND NRCALCULOCONSUMER = $nrcalculoconsumer";

        \Setforms2\Shared\Registry::getEntityManagerConnection()->exec($sql);
    }
    
    public function retornaCalculoConsumer($nrCalculoConsumer,$nrOrg){
        try{
            $db = \Setforms2\Shared\Registry::getEntityManagerConnection();
            $sql = "SELECT *
                    FROM FPA_CALCULOCONSUMER
                    WHERE
                    NRCALCULOCONSUMER = :P_NRCALCULOCONSUMER
                    AND NRORG = :P_NRORG";
            $params = array(
                'P_NRORG' => $nrOrg,
                'P_NRCALCULOCONSUMER' => $nrCalculoConsumer
            );
            return $resultado = $db->fetchAll($sql,$params);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function insereCalculoConsumer($nrorg, $nrtotvinculos, $hcmparameters, $dsordem = null, $dsinicio = null, $dsfim = null, $nrtipocalculo = null) {
        $calculoConsumer   = new \Teknisa\Model\Gestaopessoas\Entities\FpaCalculoconsumer();
        $nrcalculoconsumer = \HCM\Util\NovoCodigo::geraCodigo(0, 'FPA_CALCULOCONSUMER', 1, 20, true);
        \Setforms2\Shared\Registry::getEntityManager()->beginTransaction();
        
        try {
            $calculoConsumer->setNrcalculoconsumer($nrcalculoconsumer);
            $calculoConsumer->setNrorg($nrorg);
            $calculoConsumer->setNrtotvinculos($nrtotvinculos);
            $calculoConsumer->setIdsituacao(self::IDSITUACAO_PENDENTE);
            $calculoConsumer->setDtinclusao(DateUtil::getDataAtual());
            $calculoConsumer->setCdoperinclusao(FPA::getCdoperador());
            $calculoConsumer->setNrorginclusao($nrorg);
            $calculoConsumer->setDsOrdem($dsordem);
            
            $calculoConsumer->setNrtipocalculo($nrtipocalculo);
            if(is_array($dsinicio)){
                $dsinicio = implode(",", $dsinicio);
                $calculoConsumer->setDsInicio($dsinicio); 
            } else {
                $calculoConsumer->setDsInicio($dsinicio);
            }
            if(is_array($dsfim)){
                $dsfim = implode(",", $dsfim);
                $calculoConsumer->setDsFim($dsfim);
            } else {
                $calculoConsumer->setDsFim($dsfim);
            }
            \Setforms2\Shared\Registry::getEntityManager()->persist($calculoConsumer);
            \Setforms2\Shared\Registry::getEntityManager()->flush();
            \Setforms2\Shared\Registry::getEntityManager()->commit();
        } catch(\Exception $e) {
            \Setforms2\Shared\Registry::getEntityManager()->rollback();
            throw new \Exception($e->getMessage());
        }

        return $nrcalculoconsumer;
    }

    public function existeCalculoConsumerCriado($nrorg, $nrtotvinculos, $hcmparameters, $dsordem, $dsinicio, $dsfim, $nrtipocalculo, $tpprocessamento,$minutesAgo, $now, $maiorQueQtdeVinculos = 0) {
        try {
            $isCalculoDuplicated = false;
            
            if(!is_null($tpprocessamento) && $tpprocessamento == 'RETROCESSO'){
                return [];
            }

            if(is_array($dsinicio)){
                $dsinicio = implode(",", $dsinicio);
            }

            if(is_array($dsfim)){
                $dsfim = implode(",", $dsfim);
            }

            $db = \Setforms2\Shared\Registry::getEntityManagerConnection();

            $sql = "SELECT *
                    FROM FPA_CALCULOCONSUMER
                    WHERE
                    NRTOTVINCULOS       =   :P_NRTOTVINCULOS
                    AND NRTOTVINCULOS   >=  :P_MAIORQUEQTDVINCULOS
                    AND DSORDEM         =   :P_DSORDEM
                    AND DSINICIO        =   :P_DSINICIO
                    AND (DSFIM          =   :P_DSFIM OR (:P_DSFIM IS NULL AND DSFIM IS NULL))
                    AND DTHRINICIO      BETWEEN TO_DATE(:P_MINUTESAGO, 'DD/MM/YYYY HH24:MI:SS') 
                                           AND TO_DATE(:P_NOW, 'DD/MM/YYYY HH24:MI:SS')
                    AND IDSITUACAO      =   'ANDAMENTO'
                    AND NRORG           =   :P_NRORG
                    AND NRTIPOCALCULO   =   :P_NRTIPOCALCULO
                    AND NRTIPOCALCULO   IS NOT NULL";

            $params = array(
                'P_NRORG'               => $nrorg,
                'P_NRTOTVINCULOS'       => $nrtotvinculos,
                'P_DSORDEM'             => $dsordem,
                'P_DSINICIO'            => $dsinicio,
                'P_DSFIM'               => $dsfim,
                'P_MINUTESAGO'          => $minutesAgo->format('d/m/Y H:i:s'),
                'P_NOW'                 => $now->format('d/m/Y H:i:s'),
                'P_MAIORQUEQTDVINCULOS' => $maiorQueQtdeVinculos,
                'P_NRTIPOCALCULO'       => $nrtipocalculo
            );

            $result = $db->fetchAll($sql,$params);

            if(!empty($result)){
                return $result;
            }

            return array();
        } catch (Exception $e) {
            throw $e;
        }
    }
}