import { Component, Prop } from 'vue-property-decorator';
import { ZdComponentRender } from '@zeedhi/vuetify';
import { DialogService, CollapseCard, LoadingService } from '@zeedhi/common';
import {
  Loader, Http, IDictionary,
  I18n, Metadata, dayjs as moment,
} from '@zeedhi/core';
import { Cardapio as Info } from './common/Cardapio';

@Component({})
export default class Cardapio extends ZdComponentRender {
  @Prop({ type: Array, default: () => [] }) public cardData!: IDictionary;
  
  @Prop({ type: Array, default: () => [] }) public dates!: IDictionary;
  
  @Prop({ type: Array, default: () => [] }) public menuContent!: IDictionary;
  
  @Prop({ type: Object, default: () => ({}) }) public filterData!: IDictionary;
  
  @Prop({ type: Array, default: () => [] }) public colorCard!: IDictionary;
  
  @Prop({ type: Number, default: 3 }) public preview!: number;
  
  public instance!: Info;
  
  public utilities = Loader.getInstance('Utilities');
  
  public instanceType: typeof Info = Info;
  
  private dadosDoPrato = Loader.getInstance('Pla00115DadosDoPrato');
  
  private modalComensais = Loader.getInstance('Pme00600ComensaisEstimadoCardapio');
  
  private modalExclude = Loader.getInstance('Pme00600ExclusaoPrato');
  
  private AcompanhamentoCusto = Loader.getInstance('Pla00106AcompanhamentoCusto');
  
  private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
  
  private keyPressFunctions: IDictionary = {};

  private keyboardEnaled = false;
  
  private controlActived = false;
  
  private openingModal = false;
  
  private processDate = '';

  private currentSelectedRegister: IDictionary = {};
  
  private currentCopiedRegister: IDictionary = {};
  
  private actionEnabled = false; // Controla a navegação do cardapio.

  public created() {
    this.instance.bindNewGridAction = this.bindNewGridAction;
    this.instance.moveTableSelection = this.moveTableSelection;
    this.instance.getCurrentSelectedRegister = this.getCurrentSelectedRegister;
    this.instance.setMenuContent = this.setMenuContent;
    this.instance.forceUpdateInCard = this.forceUpdateInCard;
    this.instance.updateComens = this.updateComens;
    this.instance.dateLessDateOfProcess = this.dateLessDateOfProcess;
    this.instance.setValueAction = this.setValueAction;
    
    this.currentSelectedRegister.x = null;
    this.currentSelectedRegister.y = null;
  }

  public destroyed() {
    this.stopKeyboardHandler();
  }

  public mounted() {
    this.startKeyboardHandler();
    this.startProcessDate();
    this.makeHorizontalDefault();
  }
  
  public update() {
    this.makeHorizontalDefault();
  }
  
  public makeHorizontalDefault() {
    window.addEventListener('keydown', (e) => {
      let scroll = false;
      if ((this.currentSelectedRegister?.y === 0 || this.currentSelectedRegister?.y === null)
          && this.instance.preview === 3) {
        scroll = false;
      } else scroll = true;
      if (scroll) {
        setTimeout(() => {
          if (e.key === 'ArrowLeft' || e.key === 'ArrowRight') {
            const delta = e.key === 'ArrowLeft' ? 1 : -1;
            const plateContainer = document.getElementById('platesContainer');
            const titulosPratos = document.getElementById('titulosPratos');
            const comensais = document.getElementById('comensais');
            (plateContainer as IDictionary).scrollLeft -= delta * 100; // eslint-disable-line
            (titulosPratos as IDictionary).scrollLeft = (plateContainer as IDictionary).scrollLeft; // eslint-disable-line
            if (comensais !== null) (comensais as IDictionary).scrollLeft = (plateContainer as IDictionary).scrollLeft; // eslint-disable-line
            e.preventDefault();
          } else if (e.key === 'ArrowDown' || e.key === 'ArrowUp') {
            const delta = e.key === 'ArrowUp' ? 1 : -1;
            const cardCont = document.getElementsByClassName('card-cont')[0];
            cardCont.scrollTop += delta * 15;
            e.preventDefault();
          }
          return true;
        }, 1);
      }
    });
  }
  
  private startProcessDate() {
    this.utilities.getProcessDate().then((date: string) => {
      const format = 'DD/MM/YYYY';
      this.processDate = moment(date, format).format(format);
    });
  }
  
  private validDateBloqCard(dtIni: string, dtFin: string) {
    const format = 'DD/MM/YYYY';
    const dataIni = moment(dtIni, format);
    const dataFin = moment(dtFin, format);
    if (dataIni <= dataFin) return true; // inclusive "<="
    return false;
  }
  
  public dateLessDateOfProcess(date: string): boolean {
    return this.validDateBloqCard(this.processDate, date);
  }
  
  public isCentral(plate: IDictionary) {
    return plate.CDFILCENTPME === plate.CDFILIAL;
  }
  
  // Se a data do cardapio for menor que a data de processamento, colore o backgroud do container de vermelho claro.
  public isBlockDate(dataCardapio: string) {
    if (this.dateLessDateOfProcess(dataCardapio)) return '';
    return 'background-color: #ff000061';
  }
  
  private isHoliday(objectDate: IDictionary) {
    let color = objectDate.NRCORNUTIL;
    color = this.utilities.TColorToHex(color);
    color = color.substring(1, color.length);
    if (color.length < 6) color = this.utilities.increamentZeroInHex(color);
    
    return `color: #${color}`;
  }
  
  private increamentZeroInHex(color: string) {
    let hexColor = color;
    for (let i = 0; i < (6 - color.length); i += 1) {
      hexColor += '0';
    }
    return hexColor;
  }
  
  private isSpecific(idcardespec: string) {
    if (this.instance.filterData.aggregated !== null && idcardespec === 'S') {
      if (this.instance.colorCard.length > 0) {
        const color = this.instance.colorCard[0].NRCORCARDESP;
        const colorHex = this.utilities.TColorToHex(color);
        return `color: ${colorHex}`;
      }
    }
    return '';
  }
  
  public isSpecificOrHoliday(objectDate: IDictionary) {
    let color = '';
    if (objectDate.DTFERIFILI != null) color = this.isHoliday(objectDate);
    else color = this.isSpecific(objectDate.IDCARDESPEC);
    
    return color;
  }

  public calcGrid(size: number) {
    if (size) {
      let gtc = 'grid-gap: 2px; grid-template-columns: ';
      const cols = size;

      for (let i = 0; i < cols; i += 1) {
        gtc += '260px ';
      }
      return gtc;
    }
    return '';
  }
  
  public showValueInNumber(plate: IDictionary, e?: IDictionary) {
    let value = '';
    
    if (plate.NMPRATO && this.instance.filterData.IDPERCCOME === '%') {
      value = parseFloat(plate.VRPECONSCPME).toFixed(2);
    } else {
      const objectDate = this.instance.dates.find((data: IDictionary) => data.DTCARDIAPME === plate.DTCARDIAPME);
      if (this.elaboracaoCardapio.getFilterData.checkDinners === false) {
        value = ((plate.VRPECONSCPME / 100) * objectDate.QTCOMESTCPME).toString();
      } else {
        value = Math.round(((plate.VRPECONSCPME / 100) * objectDate.QTCOMESTCPME)).toString();
      }
      
      if (plate.NMPRATO && this.instance.filterData.IDPERCCOME === 'C' && this.instance.filterData.checkDinners) {
        value = parseFloat(value).toString();
      } else {
        // value = parseFloat(value).toFixed(3);
        value = parseFloat(value).toFixed(0);
      }
    }
    
    value = value.replace('.', ',').replace(/(\d)(?=(\d{3})+,)/g, '$1.');
    if (e === undefined) plate.valueInput = value;
    else e.target.value = value;
    // return value;
  }
  
  public formatDate(date: string) {
    const dateMonth = moment(date, 'DD/MM/YYYY').format(this.utilities.getFormatLocaleWithWeek());
    return dateMonth;
  }
  
  public dateIsHoliday(DTCARDIAPME: string) {
    const date = this.instance.dates.find((object: IDictionary) => object.DTCARDIAPME === DTCARDIAPME);
    if (date.DTFERIFILI === null) return false;
    return true;
  }
  
  public formatComens(comens: IDictionary) {
    if (comens.QTCOMESTCPME === undefined || comens.QTCOMESTCPME === null) comens.QTCOMESTCPME = 0;
    return comens.QTCOMESTCPME;
  }
  
  public openPlateDataModal(plate: IDictionary) {
    if (!this.openingModal) {
      this.openingAction(true);
      if (this.dateLessDateOfProcess(plate.DTCARDIAPME)) {
        if (!this.dateIsHoliday(plate.DTCARDIAPME)) {
          let row: IDictionary = {};
          row = this.mountDataInsertCardia(plate);
          this.insertIntoCardia(row).then(async (response) => {
            await this.elaboracaoCardapio.reloadCard();
            if (response !== undefined && response?.QTCOMESTCPME > 0) {
              const comensais = response.QTCOMESTCPME;
              const dtcardia = response.DTCARDIAPME;
              this.updateComens(comensais, dtcardia);
              plate.QTCOMESTCPME = comensais;
              this.setValueAction(true);
              this.dadosDoPrato.showModal(plate);
            } else {
              this.utilities.showAlert('dinerNull',
                'Não é possível adicionar prato, pois os comensais estão zerados.', 'error');
            }
          }).finally(() => {
            this.openingAction(false);
          });
        } else {
          this.utilities.showAlert('thisDateIsHoliday', 'Cardápio bloqueado para dia não útil.', 'error');
          this.openingAction(false);
        }
      } else {
        this.utilities.showAlert(
          'dateIsLessTheDateOfProcess',
          'Data do cardapío fora do período de processamento.',
          'error',
        );
        this.openingAction(false);
      }
    }
  }
  
  private openingAction(opening = false) {
    this.elaboracaoCardapio.showReloadCard(opening);
    this.openingModal = opening;
  }
  
  private mountDataInsertCardia(plate: IDictionary): IDictionary {
    plate.filterData = this.instance.filterData;
    const idcard = this.instance.dates.filter((data: IDictionary) => data.DTCARDIAPME === plate.DTCARDIAPME);
    plate.IDCARDESPEC = (idcard[0].IDCARDESPEC === undefined) ? 'N' : idcard[0].IDCARDESPEC;
    plate.QTCOMESTCPME = idcard[0].QTCOMESTCPME;
    const row = plate;
    row.WEEKDAY = moment(plate.DTCARDIAPME, 'DD/MM/YYYY').day();
    
    return row;
  }
  
  public insertIntoCardia(row: IDictionary) {
    return Http.post('/getCardia/get', { requestType: 'Row', row }).then((response) => {
      const responseDataCardia = response.data.dataset.cardiaOnClickPlate[0];
      return responseDataCardia;
    });
  }
  
  public changeTypeView(typeView: string) {
    this.instance.filterData.IDPERCCOME = (typeView == 'C') ? '%' : 'C'; // eslint-disable-line
    localStorage.setItem('IDPERCCOME', this.instance.filterData.IDPERCCOME);
    this.forceUpdateInCard();
  }
  
  public updateComens(comens: number, date: string) {
    this.instance.dates.forEach((data: IDictionary) => {
      if (data.DTCARDIAPME === date) {
        data.QTCOMESTCPME = comens;
      }
    });
    this.forceUpdateInCard();
  }
  
  public openModalComen(comensal: IDictionary) {
    if (!this.openingModal) {
      this.openingAction(true);
      if (!this.dateIsHoliday(comensal.DTCARDIAPME)) {
        const row = comensal;
        row.filterData = this.instance.filterData;
        LoadingService.show(' ');
        Http.post('/updateComen', { requestType: 'Row', row }).then((response: IDictionary) => {
          if (response?.data?.dataset?.hasComen?.length > 0) {
            comensal.QTCOMESTCPME = (comensal.QTCOMESTCPME === 0)
              ? response.data.dataset.hasComen[0].QTCOMESTCPME
              : comensal.QTCOMESTCPME;
            
            this.forceUpdateInCard();
            this.setValueAction(true);
            this.modalComensais.showModal(comensal);
          } else if (response?.data?.dataset?.hasComen?.length === 0) {
            this.utilities.showAlert('errorInsert', 'Dia não possui comensais estimados lançados.', 'error');
          } else {
            this.utilities.showAlert('errorInsert', 'Ocorreu um erro ao tentar inserir os comensais.', 'error');
            // console.log(response);
          }
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        }).finally(() => {
          this.openingAction(false);
          LoadingService.hide();
        });
      } else {
        this.utilities.showAlert('thisDateIsHoliday', 'Cardápio bloqueado para dia não útil.', 'error');
        this.openingAction(false);
      }
    }
  }

  public changePlateSelection(menuContentIdx: number, plateIdx: number, selection?: boolean) {
    if (plateIdx > -1) {
      const selectedPlate = this.instance.menuContent[menuContentIdx][plateIdx];
      let newSelection = selection;
      if (selectedPlate !== undefined) {
        if (newSelection === undefined) {
          newSelection = !selectedPlate.isSelected;
        }

        this.instance.menuContent[menuContentIdx].splice(
          plateIdx, 1, Object.assign(selectedPlate, { isSelected: newSelection }),
        );
        this.keyboardEnaled = true;

        if (newSelection) {
          this.currentSelectedRegister = this.instance.menuContent[menuContentIdx][plateIdx];
          this.currentSelectedRegister.x = menuContentIdx;
          this.currentSelectedRegister.y = plateIdx;
          if (this.AcompanhamentoCusto.showWindow) this.AcompanhamentoCusto.getAllDataEvenFilter();
        }
      }
    } else {
      this.changeDinersSelection(menuContentIdx, selection);
    }
  }

  private changeDinersSelection(dinerIndex: number, selection?: boolean) {
    const selectedDiner = this.instance.dates[dinerIndex];
    let newSelection = selection;
    
    if (newSelection === undefined) {
      newSelection = !selectedDiner.isSelected;
    }

    this.instance.dates.splice(dinerIndex, 1, Object.assign(selectedDiner, { isSelected: newSelection }));
    this.keyboardEnaled = true;

    if (newSelection) {
      this.currentSelectedRegister = this.instance.dates[dinerIndex];
      this.currentSelectedRegister.x = dinerIndex;
      this.currentSelectedRegister.y = -1;
      if (this.AcompanhamentoCusto.showWindow) this.AcompanhamentoCusto.getAllDataEvenFilter();
    }
  }

  public onPlateClick(plate: object, menuContentIdx: number, plateIdx: number) {
    const currentX = this.currentSelectedRegister.x;
    const currentY = this.currentSelectedRegister.y;

    if (currentX !== null && currentY !== null && !(currentX === menuContentIdx && currentY === plateIdx)) {
      this.changePlateSelection(this.currentSelectedRegister.x, this.currentSelectedRegister.y, false);
    }
    this.changePlateSelection(menuContentIdx, plateIdx);
  }
  
  public onDateClick(date: string) {
    if (this.AcompanhamentoCusto.showWindow) this.AcompanhamentoCusto.getAllDataEvenFilter(date);
  }
  
  /* Caso precisar verificar os keydown
  * https://www.w3.org/2002/09/tests/keys-cancel2.html?
  * https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/keyCode
  */
  private startKeyboardHandler() {
    window.addEventListener('keydown', this.keyboardHandler, false);
    // Setas
    this.bindNewGridAction(37, () => this.moveTableSelection('LEFT'));
    this.bindNewGridAction(38, () => this.moveTableSelection('UP'));
    this.bindNewGridAction(39, () => this.moveTableSelection('RIGHT'));
    this.bindNewGridAction(40, () => this.moveTableSelection('DOWN'));
    // Enter
    this.bindNewGridAction(13, () => this.enterKeyboardAction());
    // F4
    this.bindNewGridAction(115, () => this.excludeRegisterAction());
    this.bindNewGridAction(46, () => this.excludeRegisterAction());
    // Ctrl + C
    this.bindNewGridAction(67, () => this.copyCurrentRegisterAction());
    // Ctrl + V
    this.bindNewGridAction(86, () => this.pasteCopiedRegisterAction());
    // Ctrl + X
    this.bindNewGridAction(88, () => this.cutRegisterAction());
  }
  
  private cutRegisterAction() {
    const collapseFiltro = Metadata.getInstance<CollapseCard>('collapseCard');
    if (!collapseFiltro.expanded) {
      if (this.controlActived) {
        if (this.checkActionDisabled()) {
          if (this.currentSelectedRegister.CDPRATO !== undefined) {
            if (this.currentSelectedRegister.CDPRATO !== null) {
              if (this.dateLessDateOfProcess(this.currentSelectedRegister.DTCARDIAPME)) {
                this.copyRegister(JSON.parse(JSON.stringify(this.currentSelectedRegister)));
                this.excludeRegisterAction();
                this.pasteCopiedRegisterAction();
              } else {
                this.utilities.showAlert(
                  'dateIsLessTheDateOfProcess',
                  'Data do cardapío fora do período de processamento.',
                  'error',
                );
              }
            } else {
              this.utilities.showAlert('erroCopy', 'O local selecionado não possui nenhum registro.', 'error');
            }
          } else {
            this.utilities.showAlert('errorAction', 'Não é possivel efetuar a ação com comensais selecionada.', 'error');
          }
        }
      }
    }
  }
  
  private pasteCopiedRegisterAction() {
    const collapseFiltro = Metadata.getInstance<CollapseCard>('collapseCard');
    if (!collapseFiltro.expanded) {
      if (this.controlActived) {
        if (this.checkActionDisabled()) {
          if (Object.keys(this.currentCopiedRegister).length > 0) {
            if (this.currentSelectedRegister.CDPRATO !== undefined) {
              if (!this.dateIsHoliday(this.currentSelectedRegister.DTCARDIAPME)
                && this.dateLessDateOfProcess(this.currentSelectedRegister.DTCARDIAPME)) {
                const currentDataSave = this.currentSelectedRegister;
                this.insertRegisterCopiedInCardia(currentDataSave);
              } else {
                this.utilities.showAlert('thisDateIsHoliday', 'Cardápio bloqueado para dia não útil.', 'error');
              }
            } else {
              this.utilities.showAlert('errorAction', 'Não é possivel efetuar a ação com comensais selecionada.', 'error');
            }
          } else {
            this.utilities.showAlert('erroPaste', 'Nenhum registro foi copiado.', 'error');
          }
        }
      }
    }
  }
  
  private insertRegisterCopiedInCardia(currentDataSave: IDictionary) {
    let row: IDictionary = {};
    row = this.mountDataInsertCardia(currentDataSave);
    this.elaboracaoCardapio.showReloadCard(true);
    this.insertIntoCardia(row).then((response) => {
      if (response !== undefined && response?.QTCOMESTCPME > 0) {
        const comensais = response.QTCOMESTCPME;
        const dtcardia = response.DTCARDIAPME;
        this.updateComens(comensais, dtcardia);
        currentDataSave.QTCOMESTCPME = comensais;
        const rowSave = this.mountDataSavePlatePaste(currentDataSave);
        this.saveRegisterCopied(rowSave);
      } else {
        this.utilities.showAlert('dinerNull', 'Não é possível adicionar prato, pois os comensais estão zerados.', 'error');
      }
    }).finally(() => {
      this.elaboracaoCardapio.showReloadCard(false);
    });
  }
  
  private saveRegisterCopied(row: IDictionary) {
    this.dadosDoPrato.callSavePlateData(row).then((response: boolean) => {
      if (response) {
        this.currentSelectedRegister.CDPRATO = this.currentCopiedRegister.CDPRATO;
        this.currentSelectedRegister.NMPRATO = this.currentCopiedRegister.NMPRATO;
        this.currentSelectedRegister.CDFILAUXPRAT = this.currentCopiedRegister.CDFILAUXPRAT;
        // this.forceUpdateInCard(); /* comentado pela rotina analise de cores */
        this.elaboracaoCardapio.reloadCard();
        this.AcompanhamentoCusto.autoUpdateAcomp(this.currentSelectedRegister.DTCARDIAPME);
      }
    });
  }
  
  private mountDataSavePlatePaste(currentDataSave: IDictionary): IDictionary {
    const row: IDictionary = {};
    currentDataSave.VRPECONSCPME = this.currentCopiedRegister.VRPECONSCPME;
    row.plateData = [[currentDataSave]];
    row.selectedPlate = this.currentCopiedRegister;
    row.numberDiners = currentDataSave.QTCOMESTCPME;
    
    return row;
  }
  
  private copyCurrentRegisterAction() {
    const collapseFiltro = Metadata.getInstance<CollapseCard>('collapseCard');
    if (!collapseFiltro.expanded) {
      if (this.controlActived) {
        if (this.checkActionDisabled()) {
          if (this.currentSelectedRegister.CDPRATO !== null) {
            this.copyRegister(this.currentSelectedRegister); // == null, ele existe no objeto mas n possui dados
          } else {
            this.utilities.showAlert('erroCopy', 'O local selecionado não possui nenhum registro.', 'error');
          }
        }
      }
    }
  }
  
  private copyRegister(register: IDictionary) {
    if (register.CDPRATO !== undefined) {
      this.currentCopiedRegister = register;
    } else {
      this.utilities.showAlert('errorAction', 'Não é possivel efetuar a ação com comensais selecionada.', 'error');
    }
  }
  
  private excludeRegisterAction() {
    const collapseFiltro = Metadata.getInstance<CollapseCard>('collapseCard');
    if (!collapseFiltro.expanded) {
      const plate = this.currentSelectedRegister;
      if (this.checkActionDisabled()) {
        if (plate.CDPRATO !== undefined) {
          if (plate.CDPRATO !== null) {
            const idcard = this.instance.dates.filter((data: IDictionary) => data.DTCARDIAPME === plate.DTCARDIAPME);
            if (this.dateLessDateOfProcess(plate.DTCARDIAPME)) {
              if (this.instance.filterData.aggregated !== null && idcard[0].IDCARDESPEC !== 'S') {
                this.utilities.showAlert(
                  'dateIsLessTheDateOfProcess',
                  'Operação bloqueada! Prato não pode ser excluído pois não é específico da unidade filtrada.',
                  'error',
                );
                return;
              }
              this.setValueAction(true);
              DialogService.show({
                name: 'excludeRegisterAction',
                type: 'warning',
                title: 'Atenção',
                text: I18n.translate('Deseja excluir o prato {{PARAM1}}?', { PARAM1: plate.NMPRATO }),
                buttons: [
                  {
                    name: 'yes',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: { click: () => this.excludeRegister() },
                    outline: true,
                  },
                  {
                    name: 'no',
                    component: 'ZdButton',
                    label: 'Não',
                    events: { click: () => this.hideDialogService() },
                  },
                ],
              });
            } else {
              this.utilities.showAlert('dateIsLessTheDateOfProcess',
                'Data do cardapío fora do período de processamento.', 'error');
            }
          } else {
            this.utilities.showAlert('errorInsert', 'O local selecionado não possui nenhum registro.', 'error');
          }
        } else {
          this.utilities.showAlert('errorAction', 'Não é possivel efetuar a ação com comensais selecionada.', 'error');
        }
      }
    }
  }

  private hideDialogService() {
    this.setValueAction(false);
    DialogService.hide();
  }
  
  private excludeRegister() {
    LoadingService.show(' ');
    this.modalExclude.callExcludeRoute(this.rowExcludeKeyboard()).then(() => {
      let arrUpdate: IDictionary = [];
      this.instance.menuContent.forEach((cel: IDictionary) => {
        arrUpdate = cel.filter((plates: IDictionary) => plates.DTCARDIAPME === this.currentSelectedRegister.DTCARDIAPME);
        if (arrUpdate.length > 0) {
          arrUpdate = arrUpdate.filter((plate: IDictionary) => plate.CDPRATO !== null);
          if (arrUpdate.length <= 1) this.updateComens(0, this.currentSelectedRegister.DTCARDIAPME);
        }
      });
      this.currentSelectedRegister.CDPRATO = null;
      this.currentSelectedRegister.NMPRATO = null;
      this.currentSelectedRegister.CDFILAUXPRAT = null;
      this.currentSelectedRegister.NRCARDIAPME = null;
      this.currentSelectedRegister.VRPECONSCPME = null;
      this.hideDialogService();
      this.elaboracaoCardapio.reloadCard();
      this.AcompanhamentoCusto.autoUpdateAcomp(this.currentSelectedRegister.DTCARDIAPME);
    }).finally(() => {
      LoadingService.hide();
    });
  }
  
  private rowExcludeKeyboard(): IDictionary {
    const row: IDictionary = {};
    row.initialDateExclude = this.currentSelectedRegister.DTCARDIAPME;
    row.finalDateExclude = this.currentSelectedRegister.DTCARDIAPME;
    row.unity = (this.instance.filterData.aggregated !== null) ? 'E' : 'C';
    row.selectSpecific = (this.instance.filterData.aggregated !== null)
      ? [this.instance.filterData.aggregated.CDFILIAL] : [];
    row.platesSelect = [`${this.currentSelectedRegister.CDTIPOPRAT} - ${this.currentSelectedRegister.NRSEQCNFCPME}`];
    row.filterData = this.instance.filterData;
    return row;
  }
  
  private enterKeyboardAction() {
    const collapseFiltro = Metadata.getInstance<CollapseCard>('collapseCard');
    if (!collapseFiltro.expanded) {
      if (this.checkActionDisabled()) {
        if (Object.keys(this.currentSelectedRegister).length > 0) {
          if (this.currentSelectedRegister.CDTIPOPRAT !== undefined) this.openPlateDataModal(this.currentSelectedRegister);
          else this.openModalComen(this.currentSelectedRegister);
        }
      }
    }
  }

  private stopKeyboardHandler() {
    window.removeEventListener('keydown', this.keyboardHandler);
  }
  
  private activeControlHandler(e: KeyboardEvent) {
    this.controlActived = e.ctrlKey;
  }
  
  private keyboardHandler(e: KeyboardEvent) {
    this.activeControlHandler(e);
    if (Array.isArray(this.keyPressFunctions[`key_${e.keyCode}`]) && this.keyboardEnaled) {
      this.keyPressFunctions[`key_${e.keyCode}`].forEach((action: (e: KeyboardEvent) => void) => {
        if ([32, 37, 38, 39, 40].indexOf(e.keyCode) > -1) {
          if (document.activeElement && document.activeElement.classList
              && document.activeElement.classList[0] && document.activeElement.classList[0] !== 'input-value-plate'
          ) {
            e.preventDefault();
          }
        }
        action(e);
      });
    }
  }

  public bindNewGridAction(charKey: number, action: object) {
    if (this.keyPressFunctions[`key_${charKey}`] === undefined) {
      this.keyPressFunctions[`key_${charKey}`] = [];
    }
    this.keyPressFunctions[`key_${charKey}`].push(action);
  }

  public moveTableSelection(direction: string) {
    if (this.checkActionDisabled()) {
      const toMovePlate = Object.assign(this.currentSelectedRegister);
      const selectedElement = document.querySelector('.card-anal-container .selected') as HTMLElement;

      this.changePlateSelection(toMovePlate.x, toMovePlate.y);

      switch (direction) {
        case 'UP':
          if (this.instance.preview === 3) {
            if (toMovePlate.x > 0) toMovePlate.x -= 1;
          } else if (this.instance.preview === 2) {
            if (toMovePlate.y > 0) toMovePlate.y -= 1;
            if (toMovePlate.y === -1) toMovePlate.y = this.instance.menuContent[toMovePlate.x].length - 1;
          } else if (this.instance.preview === 1) {
            const auxMove = toMovePlate.x;
            if (toMovePlate.x !== 0) {
              if (toMovePlate.y === 0) {
                toMovePlate.x -= 1;
                toMovePlate.y = -1;
              }
            }
            
            if (toMovePlate.y > 0) toMovePlate.y -= 1;
            if (toMovePlate.y === -1 && auxMove === toMovePlate.x) {
              toMovePlate.y = this.instance.menuContent[toMovePlate.x].length - 1;
            }
          }
          break;
        case 'DOWN':
          if (this.instance.preview === 3) {
            if (toMovePlate.x < this.instance.menuContent.length - 1) toMovePlate.x += 1;
          } else if (this.instance.preview === 2) {
            toMovePlate.y += 1;
            if (toMovePlate.y >= this.instance.menuContent[toMovePlate.x].length) toMovePlate.y = -1;
          } else if (this.instance.preview === 1) {
            if (toMovePlate.y === -1) toMovePlate.x += 1;
            if (toMovePlate.x < this.instance.menuContent.length) {
              toMovePlate.y += 1;
              if (toMovePlate.y >= this.instance.menuContent[toMovePlate.x].length) toMovePlate.y = -1;
            } else {
              toMovePlate.x -= 1;
            }
          }
          break;
        case 'LEFT':
          if (this.instance.preview === 3) {
            if (toMovePlate.y > -1) toMovePlate.y -= 1;
          } else if (this.instance.preview === 2) {
            if (toMovePlate.x !== 0) toMovePlate.x -= 1;
          }
          break;
        case 'RIGHT':
          if (this.instance.preview === 3) {
            if (toMovePlate.y < this.instance.menuContent[toMovePlate.x].length - 1) toMovePlate.y += 1;
          } else if (this.instance.preview === 2) {
            if (toMovePlate.x < this.instance.menuContent.length - 1) toMovePlate.x += 1;
          }
          break;
        default:
          break;
      }

      this.changePlateSelection(toMovePlate.x, toMovePlate.y);
      
      selectedElement.scrollIntoView({
        block: 'center',
      });
    }
  }

  public getCurrentSelectedRegister(): IDictionary {
    let currRegister;

    if (this.currentSelectedRegister.y === -1) {
      currRegister = this.instance.dates[this.currentSelectedRegister.x];
    } else if (this.currentSelectedRegister.y > -1) {
      currRegister = this.instance.menuContent[this.currentSelectedRegister.x][this.currentSelectedRegister.y];
    }

    return currRegister;
  }

  public setMenuContent(menuContentValue: IDictionary) {
    this.instance.menuContent = menuContentValue;
    this.focusCurrentSelectedRegister();
    if (this.currentSelectedRegister?.x !== null && this.currentSelectedRegister?.y !== null) {
      this.changePlateSelection(this.currentSelectedRegister.x, this.currentSelectedRegister.y);
    } else this.changePlateSelection(0, 0);
    this.dadosDoPrato.filterDadosDoPrato = {};
    this.dadosDoPrato.dataGridDadosdoPrato = [];
  }
  
  public forceUpdateInCard() {
    this.$forceUpdate();
  }
  
  public focusCurrentSelectedRegister() {
    if (document.getElementsByClassName('container-max selected')[0] !== undefined) {
      (document.getElementsByClassName('container-max selected')[0] as HTMLElement).focus();
    }
  }
  
  // Verifica se tem alguma ação aberta no cardapio.
  private checkActionDisabled(): boolean {
    if (this.actionEnabled) return false;
    return true;
  }
  
  // Função utilizada para travar as ações do cardapio. OBS: precisa ta true para travar
  public setValueAction(actived: boolean) {
    this.actionEnabled = actived;
  }
  
  public changePreview(previewSelected: number) {
    this.instance.preview = previewSelected;
    const row: IDictionary = {};
    row.preview = previewSelected;
    row.filterData = this.instance.filterData;
    Http.post('/updatePreview', { requestType: 'Row', row }).then(() => {
      // this.forceUpdateInCard();
      this.elaboracaoCardapio.reloadCard();
      this.AcompanhamentoCusto.changeHeightComponentCard();
    });
  }
  
  public preventEventClickInput(e: IDictionary) {
    e.stopPropagation();
    this.setValueAction(true);
  }
  
  public preventEventBlurInput(plate: IDictionary, e: IDictionary) {
    this.setValueAction(false);
    if (this.validValueInput(e)) {
      if (plate.VRPECONSCPME !== parseFloat(e.target.value)) {
        const comensal = this.instance.dates.find(
          (date: IDictionary) => date.DTCARDIAPME.substring(0, 10) === plate.DTCARDIAPME.substring(0, 10),
        ).QTCOMESTCPME;
        if (parseFloat(comensal) > 0) {
          if (this.dateLessDateOfProcess(plate.DTCARDIAPME) && !this.dateIsHoliday(plate.DTCARDIAPME)) {
            const valueToSave = parseFloat(e.target.value.replace(/\./g, '').replace(',', '.'));
            if (this.instance.filterData.IDPERCCOME === '%') plate.VRPECONSCPME = valueToSave;
            else plate.VRPECONSCPME = ((valueToSave * 100) / comensal);
            
            const row: IDictionary = {};
            const copiedPlate = { ...plate };
            copiedPlate.filterData = this.instance.filterData;
            copiedPlate.WEEKDAY = moment(plate.DTCARDIAPME, 'DD/MM/YYYY').day();
            row.plateData = [[copiedPlate]];
            row.selectedPlate = plate;
            row.numberDiners = comensal;
            this.dadosDoPrato.callSavePlateData(row);
          }
        } else {
          e.target.value = '0,000';
          this.utilities.showAlert('dinerNull', 'Não é possível adicionar prato, pois os comensais estão zerados.', 'error');
        }
      }
    } else {
      this.showValueInNumber(plate, e);
    }
  }
  
  private validValueInput(e: IDictionary) {
    if (parseFloat(e.target.value) <= 0) {
      this.utilities.showAlert('valorMenorIgualZero', 'Não é possível inserir valores menor igual a zero.', 'error');
      return false;
    }
    if (parseFloat(e.target.value.replace(/\./g, '').replace(',', '.')) > 999999.999) {
      this.utilities.showAlert('valorMaximoUltrapassado', 'Valor máximo ultrapassado.', 'error');
      return false;
    }
    
    return true;
  }
  
  public addMaskOnKeyUp(e: IDictionary) {
    e.target.value = e.target.value.replace(/\./g, '').replace(/(\d)(?=(\d{3})+,)/g, '$1.');
  }
  
  // You can call a function on keyup event and check all the non-numeric characters and remove from the value.
  public onlyNumeric(e: IDictionary) {
    if (e.keyCode === 44) {
      if (e.target.value.includes(',')) {
        e.preventDefault();
        return false;
      }
      return true;
    }
    
    if ((e.keyCode > 31 && (e.keyCode < 48 || e.keyCode > 57)) && e.keyCode !== 46) {
      e.preventDefault();
      return false;
    }
    
    return true;
  }
}
