import {
  Loader, Metadata, Http,
  IEventParam, IDictionary, I18n, dayjs as moment, Router, Config, IRoute,
} from '@zeedhi/core';
import {
  CollapseCard, Text,
  Form, Select, DialogService,
  Progress, Date as ZdDate, Checkbox,
  Radio, Button, Row, Modal, ModalService, TextInput, LoadingService,
} from '@zeedhi/common';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';
import router from '../../../router';

export class Pme00600ElaboracaoCardapio {
  public appController = Loader.getInstance('AppController');
  
  public utilities = Loader.getInstance('Utilities');
  
  public modalController = Loader.getInstance('ModalController');
  
  public crudController = Loader.getInstance('CrudController');
  
  public showIcons = false;
  
  public showColorsAnalysis = false;
  
  public filterData: IDictionary = {};
  
  private tpCustoReal = null;
  
  private tpCustoPla = null;
  
  private idExibeCusto = null;
  
  private utilRecPadr = 'S';
  
  private idPerComensPrat = '';
  
  private leaving = false;
  
  public async createPeriodoModal() {
    this.modalController.createModal('/modals/gridPeriodo');
  }

  public destroyPeriodoModal() {
    this.modalController.destroyModal();
  }
  
  private setFilterCostType() {
    const elCustoPlanned: Select = Metadata.getInstance('planned');
    const elCustoReal: Select = Metadata.getInstance('accomplished');
    const filter: IDictionary = [];
    
    this.utilities.onMountedFilterFind(filter, 'STIDIOMA', '=', this.checkLanguage());
    
    elCustoPlanned.datasource.filter = filter;
    elCustoReal.datasource.filter = filter;
    
    elCustoPlanned.datasource.get().then((data: IDictionary) => {
      const costPlanValue = (this.tpCustoPla) ? this.tpCustoPla : data[0][elCustoPlanned.dataValue];
      
      elCustoPlanned.setValue(costPlanValue);
    });
    
    elCustoReal.datasource.get().then((data: IDictionary) => {
      const costRealValue = (this.tpCustoReal) ? this.tpCustoReal : data[0][elCustoReal.dataValue];
      
      elCustoReal.setValue(costRealValue);
    });
  }
  
  public openModalSearch() {
    // this.createPeriodoModal();
    const tipoCard: IDictionary = Metadata.getInstance('typeMenu');
    if (tipoCard.value) {
      this.modalController.openModal();
      this.utilities.setCdTipoCard(tipoCard.value.CDTIPOCARD);
      this.utilities.setFilterModalPeriodo();
    } else {
      this.utilities.showAlert('alertError', 'Tipo de Cardápio não selecionado', 'warning');
    }
  }
  
  public resetDate({ component }: IEventParam<IDictionary>) {
    component.value = null;
    this.utilities.paramsCard = null;
  }
  
  private checkLanguage() {
    const language = this.utilities.getLanguage() as string;
    let idioma = null;
    if (language != null && language.includes('en')) idioma = 'I';
    else if (language != null && language.includes('es')) idioma = 'E';
    return idioma;
  }
  
  public setFilterMenu({ component }: IDictionary) {
    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(
      filter, 'CDFILCENTPME', '=',
      this.appController.getCdCentral() ? this.appController.getCdCentral() : null,
    );
    this.utilities.onMountedFilterFind(filter, 'NRPASTATCARD', '=', null);
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterService({ component }: IEventParam<Select>) {
    const cdtipoCard = Metadata.getInstance<Select>('typeMenu');
    const selectHeaderService = Metadata.getInstance<Select>('selectHeaderService');
    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(
      filter, 'CDFILCENTPME', '=',
      this.appController.getCdCentral() ? this.appController.getCdCentral() : null,
    );
    this.utilities.onMountedFilterFind(filter, 'CDTIPOCARD', '=', cdtipoCard.value ? cdtipoCard.value.CDTIPOCARD : '');
    
    selectHeaderService.datasource.filter = filter;
    component.datasource.filter = filter;
    
    component.datasource.get();
    selectHeaderService.datasource.get();
  }
  
  public changeCardia({ component }: IEventParam<IDictionary>) {
    const menuForm = Metadata.getInstance<Form>('menuForm');
    if (this.utilities.paramsCard === null && component.value !== null) {
      const filter: IDictionary = [];
      this.utilities.onMountedFilterFind(filter, 'CDTIPOCARD', '=', menuForm.value.typeMenu.CDTIPOCARD);
      this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', this.appController.getCdCentral());
      this.utilities.onMountedFilterFind(filter, 'DTINICARDPME', '=', menuForm.value.initialDate);
      this.utilities.onMountedFilterFind(filter, 'DTFINCARDPME', '=', menuForm.value.finalDate);
      
      if (menuForm.value.initialDate !== null && menuForm.value.finalDate !== null) {
        if (menuForm.value.initialDate.length === 10 && menuForm.value.finalDate.length === 10) {
          if (this.utilities.dateIsValid(menuForm.value.initialDate, menuForm.value.finalDate)) {
            Http.post('/find/dadosCardapio', { requestType: 'FilterData', filter }).then((cardapio: IDictionary) => {
              if (cardapio.data.data.length > 0) {
                menuForm.value.NRCARDPME = cardapio.data.data[0].NRCARDPME;
                menuForm.value.describeCard = cardapio.data.data[0].DSCARDPME;
              } else {
                menuForm.value.NRCARDPME = null;
              }
            });
          }
        }
        
        if (menuForm.value.initialDate.length < 10) menuForm.value.finalDate = null;
      }
    }
  }
  
  public generateMenu() {
    const acompanhamentoCusto = Loader.getInstance('Pla00106AcompanhamentoCusto');
    if (acompanhamentoCusto.showWindow) acompanhamentoCusto.changeVisibilityWindow();
    this.verifyColorAnalysis();
    
    const menuForm = Metadata.getInstance<Form>('menuForm');
    menuForm.value.cdfilcentpme = this.appController.getCdCentral();
    if (menuForm.validate()) {
      if (this.utilities.dateIsValid(menuForm.value.initialDate, menuForm.value.finalDate)) {
        this.showReloadCard(true);
        this.utilities.getParamFilial(menuForm.value.aggregated?.CDFILIAL).then((paramfilial: IDictionary) => {
          this.utilities.getParamPlaneja(menuForm.value.aggregated?.CDFILIAL).then(
            (paramplaneja: IDictionary) => { // Pega os dados da paramplaneja
              if (this.utilities.paramsCard !== null) menuForm.value.NRCARDPME = this.utilities.paramsCard.NRCARDPME;
              menuForm.value.paramplaneja = paramplaneja;
              menuForm.value.paramfilial = paramfilial;
              this.hasConfigCard(menuForm.value);
            },
          ).catch(() => {
            this.showReloadCard(false);
          });
        }).catch(() => {
          this.showReloadCard(false);
        });
        
        Http.get('/paramgeral').then((paramgeral) => {
          this.utilRecPadr = paramgeral.data.data[0].IDUTRPADCA;
          this.idPerComensPrat = paramgeral.data.data[0].IDPERCOMENSPRAT;
        });
      } else {
        this.utilities.showAlert('invalidDate', 'Data inicial maior que a data final', 'error');
      }
    }
  }
  
  // Verifica se o cardapio ja possui configuração
  private hasConfigCard(formValue: IDictionary) {
    const row = formValue;
    
    Http.post('/hasConfigCard', { requestType: 'Row', row }).then((hasData) => {
      let perComenStorage = localStorage.getItem('IDPERCCOME');
      if (perComenStorage === null) {
        perComenStorage = hasData.data.dataset.hasConfigCard.IDPERCCOME;
        if (perComenStorage === null) perComenStorage = 'C';
      }
      row.IDPERCCOME = perComenStorage;
      
      // passa a chamar a callRulesMenu apenas se o cardápio estiver sendo criado
      if (formValue.NRCARDPME === null) { // Verifica se possui NRCARDPME
        this.callRulesMenu(formValue).then((response) => {
          if (response) {
            this.newCard(formValue);
          } else {
            this.resetAllDataCard();
            this.showReloadCard(false);
          }
        }).catch(() => {
          this.showReloadCard(false);
        });
      } else if (hasData.data.dataset.hasConfigCard.hasData) { // Verifica se tem CONFCARDPME
        this.changeDescription();
        this.openMenu(formValue).then((resolve) => { // Abre o cardapio
          if (resolve.dishes.length > 0) {
            this.setValuesHeader(formValue, resolve.colors);
            const collapse = Metadata.getInstance<CollapseCard>('collapseCard');
            if (collapse.expanded) this.toggle();
          }
        }).finally(() => {
          this.showReloadCard(false);
        });
      } else {
        this.callRulesMenu(formValue).then((response) => {
          if (response) {
            this.newCard(formValue);
          } else {
            this.resetAllDataCard();
            this.showReloadCard(false);
          }
        }).catch(() => {
          this.showReloadCard(false);
        });
      }
    }).catch(() => {
      this.showReloadCard(false);
    });
  }
  
  private newCard(formValue: IDictionary) {
    const row = formValue;
    Http.post('/card/new', { requestType: 'Row', row }).then((response) => {
      if (response.data.dataset.NRCARDPME.length > 0) {
        formValue.NRCARDPME = response.data.dataset.NRCARDPME[0];
        this.openMenu(formValue).then((resolve) => {
          if (resolve.dishes.length > 0) {
            this.setValuesHeader(formValue, resolve.colors);
            const collapse = Metadata.getInstance<CollapseCard>('collapseCard');
            if (collapse.expanded) this.toggle();
          }
        }).finally(() => {
          this.showReloadCard(false);
        });
      }
    }).catch(() => {
      this.showReloadCard(false);
    });
  }
  
  private setValuesHeader(formValue: IDictionary, colors: IDictionary[]) {
    Metadata.getInstance<Select>('selectHeaderService').setValue(formValue.service);
    Metadata.getInstance<Text>('valueHeaderTypeMenu').text = formValue.typeMenu.DESCRICAO;
    this.setTextPeriod(formValue);
    
    if (formValue.searchPlate === true) {
      Metadata.getInstance<Text>('headerReferenceDate').text = (formValue.radioSelect === 'R')
        ? 'Data de Referência para custo:'
        : 'Data do Cardápio para custo';
      
      Metadata.getInstance<Text>('valueHeaderReferenceDate').text = (formValue.radioSelect === 'R')
        ? moment(formValue.referenceDate, 'DD/MM/YYYY').format(this.utilities.getFormatLocale())
        : '';
    }
    
    if (colors.length) {
      const color = colors[0].NRCORCARDESP.toString();
      const cor = this.utilities.TColorToHex(color);
      
      const elements = document.getElementsByClassName('zd-text corAgregada');
      Array.from(elements).forEach((data: IDictionary) => {
        data.style.color = cor;
      });
    } else {
      document.getElementsByName('headerAggregated')[0].style.color = '';
      document.getElementsByName('valueHeaderAggregated')[0].style.color = 'var(--zd-font-color)';
    }
    
    if (formValue.aggregated !== null) {
      (document.getElementsByName('colAggregated')[0] as IDictionary).removeAttribute('style');
      // (document.getElementsByName('colAggregated')[0] as IDictionary).style.setProperty('width', '485px', 'important');
    } else (document.getElementsByName('colAggregated')[0] as IDictionary).setAttribute('style', 'display: none');
    Metadata.getInstance<Text>('valueHeaderAggregated').text = (formValue.aggregated !== null)
      ? formValue.aggregated.DESCRICAO : '';
  }
  
  private setTextPeriod(formValue: IDictionary) {
    const valueHeaderPeriod = Metadata.getInstance<Text>('valueHeaderPeriod');
    valueHeaderPeriod.text = moment(formValue.initialDate, 'DD/MM/YYYY').format(this.utilities.getFormatLocale());
    valueHeaderPeriod.text += ' ';
    valueHeaderPeriod.text += I18n.translate('até');
    valueHeaderPeriod.text += ' ';
    valueHeaderPeriod.text += moment(formValue.finalDate, 'DD/MM/YYYY').format(this.utilities.getFormatLocale());
    valueHeaderPeriod.text += ' - ';
    valueHeaderPeriod.text += formValue.describeCard;
  }
  
  // Pega todos os dados do menu parar poder abrir.
  public openMenu(formData: IDictionary) {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    return this.getAllDataCard(formData).then((allData) => {
      this.orderContent(allData);
      this.getWeekDay(allData.dates);
      cardapio.cardData = allData.dishes;
      cardapio.dates = this.alterComens(allData.dates, allData.comens);
      const menuContent = this.joinEqualDate(allData.dates, allData.content, allData.dishes);
      cardapio.setMenuContent(menuContent);
      cardapio.filterData = formData;
      cardapio.colorCard = allData.colors;
      cardapio.preview = (allData.preview.length > 0) ? allData.preview[0].NRVISUALCARD : 3;
      
      // Verifica se possui cardapio para exibir o icones.
      if (formData.NRCARDPME !== null) this.showIcons = true;
      else this.showIcons = false;
      
      Metadata.getInstance<CollapseCard>('collapseCard').readonly = false; // Retira o readonly do filtro.
      // Se tiver true, retira a barra de carregar
      if (Metadata.getInstance<Progress>('progressCard').isVisible) this.showReloadCard(false);
      
      this.filterData = formData;
      return allData;
    }).catch((error) => {
      console.error(error);
    });
  }
  
  private getWeekDay(dataSet: IDictionary) {
    dataSet.forEach((data: IDictionary) => {
      data.WEEKDAY = moment(data.DTCARDIAPME, 'DD/MM/YYYY').day();
    });
  }
  
  private orderContent(allData: IDictionary) {
    const newArr: IDictionary = [];
    allData.dishes.forEach((dish: IDictionary) => {
      allData.content.forEach((menu: IDictionary) => {
        if (menu.NRORDCNFCPME === dish.NRORDCNFCPME) newArr.push(menu);
      });
    });
    allData.content = newArr;
  }
  
  private alterComens(date: IDictionary, comens: IDictionary) {
    date.forEach((data: IDictionary) => {
      comens.forEach((comensal: IDictionary) => {
        if (data.DTCARDIAPME === comensal.DTCARDIAPME) {
          data.QTCOMESTCPME = comensal.QTCOMESTCPME;
          data.IDCARDESPEC = comensal.IDCARDESPEC;
        }
      });
    });
    return date;
  }
  
  private joinEqualDate(dates: IDictionary, content: IDictionary, typeDish: IDictionary) {
    const newContent: IDictionary[] = [];
    const notFound = {
      VRPECONSCPME: 0,
    };
    
    dates.forEach((date: IDictionary) => {
      const auxContent = content.filter((response: IDictionary) => date.DTCARDIAPME === response.DTCARDIAPME);
      while (auxContent.length < typeDish.length) auxContent.push(notFound);
      newContent.push(auxContent);
    });
    
    return newContent;
  }
  
  public ctrlEnterJson() {
    const collapseFiltro = Metadata.getInstance<CollapseCard>('collapseCard');
    if (collapseFiltro.expanded) {
      this.generateMenu();
    }
  }
  
  private toggle() {
    const collapse = Metadata.getInstance<CollapseCard>('collapseCard');
    collapse.expanded = !collapse.expanded;
    
    this.controllHeader(collapse.expanded);
  }
  
  public actionClickCollapse() {
    // trata para verificar se usuário está tentando mudar de um cardápio que tenha dias com comensais e sem pratos
    // caso o parâmetro idPerComensPrat = 'N' exibe mensagem
    // this.resetAllDataCard();
    // this.controllHeader();
    const fnYes = (): any => {
      const collapse = Metadata.getInstance<CollapseCard>('collapseCard');
      collapse.expanded = !collapse.expanded;
      this.resetAllDataCard();
      this.controllHeader(collapse.expanded);
      return null as any;
    };
    const fn = () => {
      const collapse = Metadata.getInstance<CollapseCard>('collapseCard');
      collapse.expanded = !collapse.expanded;
      this.resetAllDataCard();
      this.controllHeader();
    };
    const collapse = Metadata.getInstance<CollapseCard>('collapseCard');
    collapse.expanded = !collapse.expanded;
    this.openMessageComensDays(fnYes, fn);
  }
  
  // Função para controlar os dados exibidos no header. Necessario saber se o header foi aberto pelo toogle ou por click
  public controllHeader(expanded?: boolean) {
    const collapseCard = Metadata.getInstance<CollapseCard>('collapseCard');
    const menuForm = Metadata.getInstance<Form>('menuForm');
    if (!collapseCard.readonly) {
      const showCollapse = (typeof expanded !== 'boolean') ? !collapseCard.expanded : expanded;
      
      const formHeaderDisplay = Metadata.getInstance<Row>('headerForm');
      const collapseHeader = Metadata.getInstance<Row>('collapseCardForm');
      const contentForm = Metadata.getInstance<Form>('contentForm');
      
      if (menuForm.validate() && menuForm.value.NRCARDPME !== null) {
        if (showCollapse) {
          collapseHeader.isVisible = true;
          formHeaderDisplay.isVisible = false;
          contentForm.isVisible = false;
        } else {
          collapseHeader.isVisible = false;
          formHeaderDisplay.isVisible = true;
          contentForm.isVisible = true;
        }
      } else {
        this.resetAllDataCard();
      }
    }
  }
  
  private getCssHeader(hasShow: boolean) {
    if (hasShow) return 'initial';
    return 'none';
  }
  
  /* Regras especificas do cardapio */
  /* Responsavel por verificar todas as regras do cardapio.
     (true) pode continuar a insercao
     (false) para de verificar para exibir a mensagem e nao pode inserir.
  */
  private callRulesMenu(form: IDictionary) {
    const row = form;
    return Http.post('/menuRules', { requestType: 'Row', row }).then((response) => {
      const message = response.data.dataset.message;
      if (!message.isCorrect) this.utilities.showAlert('callRuleError', message.message, 'error');
      
      return message.isCorrect;
    });
  }
  
  // Pega nos nomes dos tipos de pratos.
  private getAllDataCard(component: IDictionary) {
    const row = component;
    return Http.post('/card/get', { requestType: 'Row', row }).then((response) => {
      const allData = response.data.dataset.allData;
      allData.dishes.forEach((dish: IDictionary) => {
        if (dish.IDOPCCNFCPME === 'S') dish.NMTIPOPRAT += ' (Opção)';
      });
      return allData;
    });
  }
  
  public updateComen({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    const menuForm = Metadata.getInstance<Form>('menuForm');
    if (menuForm.value.NRCARDPME !== undefined) {
      this.utilities.getProcessDate().then((processDate: string) => {
        if (menuForm.value.aggregated !== null) {
          DialogService.show({
            name: 'pme00600_atualizarcomensais',
            type: 'warning',
            title: 'Atenção',
            text: 'Deseja atualizar os comensais?',
            buttons: [
              {
                name: 'yes',
                component: 'ZdButton',
                label: 'Sim',
                events: { click: () => this.updateComenAggregated(menuForm.value, processDate) },
                outline: true,
              },
              {
                name: 'no',
                component: 'ZdButton',
                label: 'Não',
                events: { click: () => DialogService.hide() },
              },
            ],
          });
        } else {
          const modalAtualizaComensal = Loader.getInstance('Pme00600ModalAtualizaComensal');
          modalAtualizaComensal.showModal(menuForm.value, processDate);
        }
      });
    } else {
      this.utilities.showAlert('withoutFilter', 'Operação bloqueada! Favor realizar o filtro.', 'error');
    }
  }
  
  private updateComenAggregated(valueForm: IDictionary, processDate: string) {
    const row = valueForm;
    row.DATAPROCE = processDate;
    Http.post('/procedureUpdateComen', { requestType: 'Row', row }).then((response) => {
      if (response.data.dataset.updated[0]) this.utilities.showAlert('updated', 'Atualizado com sucesso!', 'success');
      DialogService.hide();
      this.reloadCard();
    }).catch((error: string) => {
      DialogService.hide();
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }
  
  // Atualiza o cardapio (Refaz a requisição dos dados filtrados)
  public reloadCard(isColorAnalysis = false) {
    const menuForm = Metadata.getInstance<Form>('menuForm');
    this.openMenu(menuForm.value);
    this.showReloadCard(true);
    this.verifyColorAnalysis(isColorAnalysis);
  }
  
  // Essa função é chamada apenas quando é clicada com o mouse ou utilizada a keyMap.
  public reloadCardClickIcon({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    const acompanhamentoCusto = Loader.getInstance('Pla00106AcompanhamentoCusto');
    if (acompanhamentoCusto.showWindow) acompanhamentoCusto.changeVisibilityWindow();
    this.reloadCard();
  }
  
  public showReloadCard(show = false) {
    const reload = Metadata.getInstance<Progress>('progressCard');
    const btnGenerateMenu = Metadata.getInstance<Button>('generateMenu');
    if (show !== null || show !== undefined) reload.isVisible = show;
    else reload.isVisible = !reload.isVisible;
    
    if (show) btnGenerateMenu.disabled = true;
    else btnGenerateMenu.disabled = false;
  }
  
  public disableDate({ component }: IEventParam<Checkbox>) {
    const radioDate = Metadata.getInstance<Radio>('radioSelect');
    
    if (component.value) {
      radioDate.disabled = false;
    } else {
      radioDate.disabled = true;
      radioDate.value = 'C';
      Metadata.getInstance<ZdDate>('referenceDate').disabled = true;
      Metadata.getInstance<ZdDate>('referenceDate').value = '';
    }
    
    this.changeStyleHeaderReference(component.value);
    this.firstDataRadio();
    if (radioDate.value === 'R' && component.value) this.ableReferenceDate();
    this.validateReferenceDate(radioDate.value);
  }
  
  private firstDataRadio() {
    const radioDate: IEventParam<Radio> = Metadata.getInstance('radioSelect');
    if (radioDate.value === undefined) radioDate.value = (radioDate.datasource as IDictionary).data[0].value;
  }
  
  public ableReferenceDate() {
    const radioDate = Metadata.getInstance<Radio>('radioSelect');
    const referenceDate = Metadata.getInstance<ZdDate>('referenceDate');
    
    if (radioDate.value === 'R') {
      referenceDate.disabled = false;
      referenceDate.value = moment(moment(), 'DD/MM/YYYY').format(this.utilities.getFormatLocale());
    } else {
      referenceDate.disabled = true;
      referenceDate.value = '';
    }
    
    this.validateReferenceDate(radioDate.value);
  }
  
  private validateReferenceDate(radioDate: string) {
    const referenceDate = Metadata.getInstance<ZdDate>('referenceDate');
    const searchPlate = Metadata.getInstance<Checkbox>('searchPlate');
    if (radioDate === 'R' && searchPlate.value) {
      (referenceDate as IDictionary).addValidation('required', { message: 'Campo obrigatório' });
    } else {
      (referenceDate as IDictionary).rules = [];
      Metadata.getInstance<ZdDate>('referenceDate').disabled = true;
    }
  }
  
  // Se "exibir custos..." estiver selecionado irá aparecer no header
  private changeStyleHeaderReference(hasShow: boolean) {
    if (hasShow) {
      (document.getElementsByName('colReferenceDate')[0] as IDictionary).removeAttribute('style');
      (document.getElementsByName('colReferenceDate')[0] as IDictionary).style.setProperty('width', '485px', 'important');
    } else (document.getElementsByName('colReferenceDate')[0] as IDictionary).setAttribute('style', 'display: none');
  }
  
  public changeStateBaseCost({ component }: IEventParam<Select>) {
    const baseCost = Metadata.getInstance<Checkbox>('baseCost');
    if (component.value) baseCost.disabled = false;
    else {
      baseCost.disabled = true;
      baseCost.value = true;
    }
  }
  
  public filterAggregated({ component }: IEventParam<Select>) {
    const cdtipoCard = Metadata.getInstance<Select>('typeMenu');
    const service = Metadata.getInstance<Select>('service');

    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', this.appController.getCdCentral());
    this.utilities.onMountedFilterFind(filter, 'CDTIPOCARD', '=', cdtipoCard.value?.CDTIPOCARD);
    this.utilities.onMountedFilterFind(filter, 'CDSERVFILI', '=', service.value?.CDSERVFILI);
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public clearFields({ component }: IEventParam<Select>) {
    const servico = Metadata.getInstance<Select>('service');
    const menuForm = Metadata.getInstance<Form>('menuForm');
    
    if (component.name === 'typeMenu') {
      servico.setValue('');
      menuForm.value.NRCARDPME = null;
      menuForm.value.describeCard = null;
      menuForm.value.finalDate = null;
      menuForm.value.initialDate = null;
    }
  }
  
  public get getFilterData() {
    if (this.filterData.referenceDate === null) this.filterData.referenceDate = ''; // em caso de ser null
    return this.filterData;
  }
  
  public cancelEventHeader({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
  }
  
  public changeServiceFilter({ component }: IEventParam<Select>) {
    // trata para verificar se usuário está tentando mudar de um cardápio que tenha dias com comensais e sem pratos
    // caso o parâmetro idPerComensPrat = 'N' exibe mensagem
    const fnYes = () => {
      const service = Metadata.getInstance<Select>('service');
      if (component.value !== null) {
        if (service.value?.CDSERVFILI !== component.value?.CDSERVFILI) {
          service.setValue(component.value);
          (document.getElementsByName('selectHeaderService')[0] as HTMLElement).blur(); // Retira o foco do campo
          this.showReloadCard(true);
          this.generateMenu();
        }
      } else {
        service.setValue(component.value);
        (document.getElementsByName('selectHeaderService')[0] as HTMLElement).blur(); // Retira o foco do campo
        this.reloadCard();
      }
    };
    const fnNo = () => {
      const service = Metadata.getInstance<Select>('service');
      component.value = service.value;
      DialogService.hide();
    };
    const service = Metadata.getInstance<Select>('service');
    // checa se cardápio está sendo gerado pela primeira vez para validar os dias sem comensais
    if (service.value?.CDSERVFILI !== component.value.CDSERVFILI) this.openMessageComensDays(fnYes, fnYes, fnNo);
    else fnYes();
  }

  public clearAgreg() {
    const agreg = Metadata.getInstance<Select>('aggregated');
    const collapse = Metadata.getInstance<CollapseCard>('collapseCard');
    if (!collapse.expanded && agreg.value !== null) {
      this.utilities.showAlert('backToCentral', 'Atenção! Cardápio da central carregado.', 'warning');
    }
    agreg.setValue('');
  }
  
  private resetAllDataCard() {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    cardapio.cardData = [];
  }
  
  private openMessageComensDays(fnYes: () => any, fn: () => any, fnNo: () => any = () => { DialogService.hide(); }) {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    if (this.idPerComensPrat === 'N' && cardapio.cardData.length > 0) {
      const days = this.checkForDayWithComenWithoutPlates(cardapio);
      if (days.length > 0) {
        const daysConcat = days.map((el: any) => moment(el.DTCARDIAPME, 'DD/MM/YYYY')
          .format(Config.displayFormat)).join(', ');
        this.showMessageComensDays(daysConcat, fnYes, fnNo);
      } else fn();
    } else fn();
  }
  
  private showMessageComensDays(daysConcat: string, func: () => any, funcNo: () => any = () => { DialogService.hide(); }) {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    DialogService.show({
      name: 'excluiComensDiasSemPrato',
      type: 'warning',
      title: 'Atenção',
      text: `Cardápio possui dia com comensal mas sem prato informado.<br>${daysConcat}`
            + '<br>Estes serão apagados. Deseja continuar?',
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          outline: true,
          events: {
            click: () => {
              this.deleteComensInDaysWithoutPlate(cardapio).then(func());
            },
          },
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: {
            click: () => {
              funcNo();
            },
          },
        },
      ],
    });
  }
  
  public plateColorAnalysis({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    
    const elements = document.getElementsByClassName('container-max');
    this.showColorsAnalysis = !this.showColorsAnalysis;
    
    if (this.showColorsAnalysis === true) this.enableColorAnalysis(elements, true);
    else this.disableColorAnalysis(elements);
  }
  
  private enableColorAnalysis(elements: ArrayLike<Element> = [], needsReload = false) {
    if (needsReload) this.reloadCard();
    if (!this.showColorsAnalysis) this.showColorsAnalysis = true;
    
    const arrayElements = (elements.length > 0) ? elements : document.getElementsByClassName('container-max');
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    const menuContent = cardapio.menuContent.flat();
    const defaultColor = 'rgb(240, 240, 240)';
    // const whiteColor = '#ffffff';
    // const blackColor = '#000000';
    const emptyString = '';
    
    // na Visualizacao 1 deve-se remover alguns elementos
    const newArrayElements = (cardapio.preview === 1)
      ? Array.from(arrayElements).filter((value: IDictionary) => (Number.isNaN(parseFloat(value.innerText))))
      : arrayElements;

    // os arrays newArrayElements e menuContent devem estar "sincronizados"
    Array.from(newArrayElements).forEach((data: IDictionary, index) => {
      const currentPlateInfo = menuContent[index];
      const colorString = (currentPlateInfo.NRCOR) ? currentPlateInfo.NRCOR.toString() : emptyString;
      const color = (colorString === emptyString) ? defaultColor : this.utilities.TColorToHex(colorString);
      
      if ((data.innerText !== emptyString) && (color !== defaultColor)) {
        // const newInnerTextColor = (color === blackColor) ? whiteColor : blackColor;
        // eslint-disable-next-line no-bitwise
        // const invertColor = (Number(`0x1${color.substring(1)}`) ^ 0xFFFFFF).toString(16).substr(1).toUpperCase();
        const newInnerTextColor = this.invertColor(color, true);
        
        // data.style.backgroundColor = color;
        data.style.setProperty('background-color', color, 'important');
        data.getElementsByClassName('cell')[0].style.color = newInnerTextColor;
        data.getElementsByTagName('input')[0].style.color = newInnerTextColor;
        data.getElementsByClassName('palate-data')[0].style.opacity = 1;
      }
    });
  }
  
  private padZero(str: string, len: number = 2) {
    const zeros = new Array(len).join('0');
    return (zeros + str).slice(-len);
  }
  
  private invertColor(hexP: string, bw: boolean) {
    let hex = hexP;
    if (hex.indexOf('#') === 0) {
      hex = hex.slice(1);
    }
    // convert 3-digit hex to 6-digits.
    if (hex.length === 3) {
      hex = hex[0] + hex[0] + hex[1] + hex[1] + hex[2] + hex[2];
    }
    if (hex.length !== 6) {
      throw new Error('Invalid HEX color.');
    }
    const r = parseInt(hex.slice(0, 2), 16);
    const g = parseInt(hex.slice(2, 4), 16);
    const b = parseInt(hex.slice(4, 6), 16);
    if (bw) {
      return (r * 0.299 + g * 0.587 + b * 0.114) > 186
        ? '#000000'
        : '#FFFFFF';
    }
    const rS = (255 - r).toString(16);
    const gS = (255 - g).toString(16);
    const bS = (255 - b).toString(16);
    return `#${this.padZero(rS)}${this.padZero(gS)}${this.padZero(bS)}`;
  }

  private disableColorAnalysis(elements: ArrayLike<Element> = []) {
    if (this.showColorsAnalysis) this.showColorsAnalysis = false;
    
    const defaultColor = 'rgb(240, 240, 240)';
    const opacity = 0.6;
    const emptyString = '';
    const arrayElements = (elements.length > 0) ? elements : document.getElementsByClassName('container-max');
    
    Array.from(arrayElements).forEach((data: IDictionary) => {
      if (data.innerText !== emptyString && data.title !== emptyString) {
        data.style.backgroundColor = defaultColor;
        data.getElementsByClassName('cell')[0].style.color = emptyString;
        data.getElementsByTagName('input')[0].style.color = emptyString;
        data.getElementsByClassName('palate-data')[0].style.opacity = opacity;
      }
    });
  }
  
  private verifyColorAnalysis(isColorAnalysis = false) {
    if (!isColorAnalysis) this.disableColorAnalysis();
    else this.enableColorAnalysis();
  }
  
  private getParamPlanejaPadroes() {
    this.utilities.getParamPlanejaPadroes(this.appController.cdCentral).then((paramplaneja: IDictionary) => {
      this.tpCustoReal = paramplaneja?.CDTIPOCUSTOREA;
      this.tpCustoPla = paramplaneja?.CDTIPOCUSTOPLA;
      this.idExibeCusto = paramplaneja?.IDEXIBECUSTO;
      
      this.setFilterCostType();
      this.setSearchPlate();
    });
  }
  
  private setSearchPlate() {
    const elSearchPlate: Checkbox = Metadata.getInstance('searchPlate');
    const elRadioSelect: Checkbox = Metadata.getInstance('radioSelect');
    const elreferenceDate: ZdDate = Metadata.getInstance('referenceDate');
    
    if (this.idExibeCusto === 'C') {
      elSearchPlate.value = true;
      elRadioSelect.disabled = false;
      elRadioSelect.value = 'C';
      elreferenceDate.disabled = true;
    } else if (this.idExibeCusto === 'R') {
      elSearchPlate.value = true;
      elRadioSelect.disabled = false;
      elRadioSelect.value = 'R';
      elreferenceDate.value = moment(moment(), 'DD/MM/YYYY').format(this.utilities.getFormatLocale());
    }
  }
  
  public changeDescription() {
    if (this.utilities.description !== this.utilities.descCard) {
      // const describeCard = Metadata.getInstance<TextInput>('describeCard');
      const row = {
        DSCARDPME: this.utilities.descCard,
        CDFILCENTPME: this.appController.getCdCentral(),
        NRCARDPME: this.utilities.nrCardPme,
        CDTIPOCARD: this.utilities.cdtipoCard,
      };
      
      Http.post('/saveCardapioDescription', { requestType: 'Row', row }).then(() => {
        this.utilities.description = this.utilities.descCard;
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
  
  private modalSearch!: Modal;
  
  private async createModalSearch() {
    this.modalSearch = await ModalService.createFromJsonPost('',
      { jsonPath: 'pme/cardapio/modals/searchCardapio' });
  }
  
  public showSearch({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    if (!this.searchApplied) {
      this.openSearch();
    } else if (this.searchApplied) {
      if (event?.type === 'click') {
        this.disableSearch();
        this.searchApplied = false;
      } else {
        this.disableSearch();
        this.searchApplied = false;
        this.openSearch();
      }
    }
  }
  
  private openSearch() {
    if (this.utilities.checkOpenModalOverlap()) {
      if (!this.modalSearch) {
        this.createModalSearch().then(() => {
          this.modalSearch.show();
        });
      } else {
        this.modalSearch.show();
      }
    }
  }
  
  public cancelModalSearch() {
    if (this.modalSearch) this.modalSearch.destroy();
    this.modalSearch = this.utilities.modalEmpty;
  }
  
  private searchApplied: boolean = false;
  
  public get iconSearch() {
    return this.searchApplied ? 'mdi-magnify-close' : 'mdi-magnify';
  }
  
  public get searchIconName() {
    return this.searchApplied ? 'Cancelar Pesquisa' : 'Pesquisar';
  }
  
  private disableSearch() {
    const searchElement = document.querySelectorAll('.cell');
    searchElement.forEach((el) => {
      el.classList.remove('highlight');
    });
  }
  
  public executeSearch() {
    const searchText = Metadata.getInstance<TextInput>('searchText');
    const searchElement = document.querySelectorAll('.cell');
    if (searchText.value) {
      let scrollEl: any = null;
      searchElement.forEach((el) => {
        if (this.utilities.strtr(el.innerHTML.toLowerCase())
          .includes(this.utilities.strtr(searchText.value.toLowerCase()))) {
          if (scrollEl === null) scrollEl = (el.getBoundingClientRect().top - 110) + window.scrollY;
          el.classList.add('highlight');
        }
      });
      if (scrollEl !== null) {
        window.scroll({
          top: scrollEl,
          behavior: 'smooth',
        });
      }
      this.searchApplied = true;
    }
    this.cancelModalSearch();
  }
  
  private checkForDayWithComenWithoutPlates(component: IDictionary) {
    const datesWithComens = component.dates.filter((date: IDictionary) => (date.QTCOMESTCPME > 0));
    
    return datesWithComens.filter((date: IDictionary) => {
      const index = component.dates.findIndex((data: IDictionary) => (data.DTCARDIAPME === date.DTCARDIAPME));
      const platesInDay = component.menuContent[index]
        .filter((plate: IDictionary) => (plate.DTCARDIAPME === date.DTCARDIAPME && plate.CDPRATO !== null));
      if (platesInDay.length === 0) return true;
      return false;
    });
  }
  
  private async deleteComensInDaysWithoutPlate(cardapio: IDictionary) {
    LoadingService.show(' ');
    return Http.post('/deleteComensInDaysWithoutPlate',
      { requestType: 'Row', row: cardapio.filterData }).then((response) => {
      if (response.data.dataset.updated[0]) this.utilities.showAlert('updated', 'Atualizado com sucesso!', 'success');
    }).catch((error: string) => {
      this.reloadCard();
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      DialogService.hide();
      LoadingService.hide();
    });
  }
  
  public setListenerRouteLeave() {
    // adiciona evento para tratar caso o usuário esteja com cardápio aberto e clique pra mudar de tela no menu
    // caso o cardápio aberto tenha dias com comensais e sem prato, e o parâmetro idPerComensPrat = 'N' exibe a mensagem
    router.beforeEach((to, from, next) => {
      if (from.path === '/pme00600_elaboracao_cardapio') {
        if (this.leaving) next();
        else {
          const fnYes = (): any => {
            this.leaving = true;
            Router.replace(to as IRoute);
            return null as any;
          };
          this.openMessageComensDays(fnYes, fnYes);
        }
      } else next();
    });
    
    if (this.headerFixed) {
      const card = document.querySelector('#collapseCard');
      card?.classList.add('fixCardHeader');
    }
  }
  
  private headerFixed: boolean = localStorage.getItem('HEADER_FIXED_CARD') === 'true';
  
  public get iconFixHeader() {
    return this.headerFixed ? 'mdi-pin-off' : 'mdi-pin';
  }
  
  public get fixHeaderIconName() {
    return this.headerFixed ? 'Cancelar Fixar Cabeçalho' : 'Fixar Cabeçalho';
  }
  
  public toggleFixHeaderCard({ event }: IEventParam<Button>) {
    const card = document.querySelector('#collapseCard');
    card?.classList.toggle('fixCardHeader');
    const labels = document.querySelector('.fixed-labels-fixed-header');
    if (labels !== null) {
      labels.classList.remove('fixed-labels-fixed-header');
      labels.classList.add('fixed-labels');
    }
    // se estiver cancelando fixamento do cabeçalho, retirar propriedade para calcular
    // novamente no arquivo visualizacao 3 método handleScroll
    if (this.headerFixed) (labels as HTMLElement)?.style.removeProperty('top');
    this.headerFixed = !this.headerFixed;
    if (this.headerFixed) localStorage.setItem('HEADER_FIXED_CARD', 'true');
    else localStorage.setItem('HEADER_FIXED_CARD', 'false');
    event?.preventDefault();
    event?.stopPropagation();
  }
  
  public get showFixHeaderCard() {
    return this.showIcons && (Metadata.getInstance('cardapio') as Cardapio)?.preview === 3;
  }
  
  public checkCloseActions({ component }: IEventParam<CollapseCard>) {
    if (component.expanded) {
      const acompanhamentoCusto = Loader.getInstance('Pla00106AcompanhamentoCusto');
      if (acompanhamentoCusto.showWindow) acompanhamentoCusto.changeVisibilityWindow();
    }
  }
}
