import {
  Http,
  IEventParam,
  Loader,
  Metadata,
  Config,
  IDictionary,
} from '@zeedhi/core';
import {
  Form, Checkbox, Button, Select, Increment, Modal, ModalService, SpeedDial, Radio, LoadingService,
} from '@zeedhi/common';
import { PmeMask } from '@/config';
  
export class Pme00900RequisicaoServAnalController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');
  
  public modalController = Loader.getInstance('ModalController');

  public utilities = Loader.getInstance('Utilities');
  
  public modalHistorico!: Modal;
  
  public blockActivator = false;

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }
  
  public async createModalHistorico() {
    if (!this.modalHistorico) {
      this.modalHistorico = await ModalService.createFromJsonPost('',
        { jsonPath: 'pme/requisicao/modals/pme03800_historico_requisicao_analitica' });
    }
  }
  
  public cancelModalHistorico() {
    this.modalHistorico.destroy();
  }
  
  public openModalHistorico() {
    this.createModalHistorico().then(() => {
      this.modalHistorico.show();
    });
  }

  public disableLinhaBranca() {
    const linhaBranco = Metadata.getInstance<Checkbox>('LINHASBRANCO');
    const numLinhaBranco = Metadata.getInstance<Increment>('NRLINHABRANCA');
    
    if (linhaBranco.value === true) numLinhaBranco.disabled = false;
    else {
      numLinhaBranco.disabled = true;
      numLinhaBranco.value = 0;
    }
  }

  public calculateCons() {
    const form: Form = Metadata.getInstance<Form>('FORMREQSERVANAL');
    const elActions = Metadata.getInstance<SpeedDial>('actionsRequisicao');

    if (this.utilities.centralIsValid() && this.formIsValid(form)) {
      LoadingService.show();
      this.blockActivator = true;
      const row = form.value;
      row.CDFILCENTPME = this.appController.getCdCentral();
      row.CDARVPRODINI = row.PRODINICIAL ? row.PRODINICIAL.CDARVPROD : null;
      row.CDARVPRODFIN = row.PRODFINAL ? row.PRODFINAL.CDARVPROD : null;
      if (!row.CDFILCENTPME) {
        this.utilities.showAlert('alertError', 'Central não informada.', 'error');
        LoadingService.hide();
      } else {
        this.utilities.activeLoadingRoundButton(elActions);
        Http.post('calculate', { requestType: 'Row', row }).then(() => {
          this.utilities.showAlert('alertSuccess', 'Calculado com Sucesso.', 'success');
        }).finally(() => {
          this.utilities.disableLoadingRoundButton(elActions);
          this.blockActivator = false;
          LoadingService.hide();
        });
      }
    }
  }
  
  public activatorClick({ event }: IEventParam<Button>) {
    if (this.blockActivator) event?.stopPropagation();
  }
  
  private formIsValid(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    if (form.validate()) {
      if (this.utilities.dateIsValid(row.DTDE, row.DTATE)) {
        const prodIni = row.PRODINICIAL;
        const prodFin = row.PRODFINAL;
        if ((prodIni == null || prodFin == null)
            || this.utilities.prodIsValid(prodIni.CDARVPROD, prodFin.CDARVPROD)
        ) {
          valid = true;
        } else this.utilities.showAlert('alertError', 'Produto inicial maior que o final.', 'error');
      } else this.utilities.showAlert('alertError', 'Data Inicial deve ser menor que a data final.', 'error');
    }
    return valid;
  }
  
  public setFilterAgregAgrup({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnid.value ? agrupUnid.value.CDGRUPFILI : 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
 
  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORMREQSERVANAL');
    
    if (this.utilities.centralIsValid() && this.formIsValid(form)) {
      const row: IDictionary = form.value;
      if (row.GRUPFILI != null) {
        row.CDGRUPFILI = row.GRUPFILI.CDGRUPFILI;
        row.NMGRUPFILI = row.GRUPFILI.NMGRUPFILI;
      } else {
        row.CDGRUPFILI = null;
        row.NMGRUPFILI = null;
      }
      
      this.setParams(row);
      if (!row.CDFILCENTPME) this.utilities.showAlert('alertError', 'Central não informada.', 'error');
      else {
        this.reportController.callGenerateReport(component, row, 'report/requisicaoAnalServ');
      }
    }
  }
  
  private setValueQuebra(row: IDictionary) {
    const quebra = Metadata.getInstance<Checkbox>('QUEBRAPAG').value;
    
    if (quebra === false) row.QUEBRAPAG = false;
    
    else row.QUEBRAPAG = true;
  }
  
  private setParams(row: IDictionary) {
    const mask: string = (Config.masks.productMask as PmeMask).pattern;
    this.setValueQuebra(row);
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.DTPREVCONPME = row.DTPREVCONPME ? row.DTPREVCONPME.DTPREVCONPME : null;
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    row.TIPOCARD = row.TIPOCARD ? row.TIPOCARD : null;
    row.UNIDADE = row.UNIDADE ? row.UNIDADE : null;
    row.SERVICO = row.SERVICO ? row.SERVICO : null;
    row.NRLISTA = row.NRLISTA ? row.NRLISTA.NRLISTA : null;
    row.TAMNIV = this.utilities.calcTamacNivel(row.NIVEL, 'produto');
    row.CDPICTPROD = this.utilities.formatReportMask(mask);
    row.CDARVPRODINI = row.PRODINICIAL ? row.PRODINICIAL.CDARVPROD : null;
    row.CDARVPRODFIN = row.PRODFINAL ? row.PRODFINAL.CDARVPROD : null;
    row.NIVEL = row.NIVEL !== undefined ? row.NIVEL : 0;
    row.ACRESCIMO = row.ACRESCIMO !== undefined ? row.ACRESCIMO : 0;
    row.NRLINHABRANCA = row.NRLINHABRANCA !== undefined ? row.NRLINHABRANCA : 0;
  }

  public get disableNivel() {
    const ordena = Metadata.getInstance<Radio>('ORDENA').value;
    return ordena !== '1';
  }
}
